"""List exercises"""


def combine_lists():
    """Return a new list that combines the two given lists."""


def glink():
    """Convert Google Drive file URL to download URL."""


def rotate_list():
    """Move first list item to end of list and return the item."""


def characters():
    """Return list of all characters in given string."""


def reverse_words():
    """Return the given sentence with the words in reverse order."""


def ith_item_power():
    """Returns i-th element raised to the i-th power."""


def natural_sort():
    """From list of words, return new list of words in natural sorted order."""
